#! /bin/bash
#
# Author Markus Gruenwald
# Date 2010-03-11
#
#
#
 
 
PROGNAME="${0##*/}"
PROGPATH="${0%/*}"
VERSION="20120821"
 
. $PROGPATH/utils.sh
 
 
WARNING=30
CRITICAL=60
 
 
 
print_usage() {
    echo ""
    echo "Usage:"
    echo ""
    echo "$PROGNAME -i <IP-Adress>"
    echo ""
    echo "$PROGNAME -w THRESHOLD"
    echo "    Offset to result in warning status (seconds)"
    echo "    Default is $WARNING sec"
    echo ""
    echo "$PROGNAME -c THRESHOLD"
    echo "    Offset to result in critical status (seconds)"
    echo "    Default is $CRITICAL sec"
    echo ""
    echo "$PROGNAME --help"
    echo "$PROGNAME --version"
    echo ""
}
 
print_help() {
    print_head
    echo ""
    print_usage
    echo ""
    echo "This plugin checks the time difference between local time and the time on a windows host. It use the net tool that is part of the part of samba-client."
    echo ""
}
 
 
print_head() {
	echo "$PROGNAME Version $VERSION"
        $ECHO "This plugin come with ABSOLUTELY NO WARRANTY. You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\n"
}
 
 
verify_ip() {
        if echo $1|egrep "^([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])\.([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])\.([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])\.([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])$" > /dev/null 2>&1
          then
            VERIFY_IP=0
          else
            VERIFY_IP=1
        fi
}
 
STATUS=0
! NET_BIN=$(which net 2>/dev/null) && echo "ERROR: MISSING net (part of samba-client)" && exit $STATE_UNKNOWN
 
if [[ "$STATUS" == "1" ]]
  then
    echo ""
    echo "ERROR: Failed dependencies ..."
    echo ""
    exit 1
 
fi
 
 
# Make sure the correct number of command line
# arguments have been supplied
 
if [ $# -lt 1 ];
  then
    echo "Can not find any argument"
    print_usage
    exit $STATE_UNKNOWN
fi
 
# Grab the command line arguments
 
while test -n "$1"; do
    case "$1" in
        --help)
            print_help
            exit $STATE_OK
            ;;
        -h)
            print_help
            exit $STATE_OK
            ;;
        --version)
            print_head
            exit $STATE_OK
            ;;
        -V)
            print_head
            exit $STATE_OK
            ;;
        -i)
            IPADDR=$2
            verify_ip $IPADDR
 
            if [[ "$VERIFY_IP" != "0" ]]
              then
                echo "Invalid ip-address: $IPADDR"
                print_usage
                exit $STATE_UNKNOWN
            fi
 
            shift
            ;;
        -w)
            WARNING=$2
            shift
            ;;
        -c)
            CRITICAL=$2
            shift
            ;;
        *)
            echo "Unknown argument: $1"
            print_usage
            exit $STATE_UNKNOWN
            ;;
    esac
    shift
done
 
if (( $WARNING > $CRITICAL ))
  then
    echo "WARNING ($WARNING) cannot be larger than CRITICAL ($CRITICAL)"
    exit $STATE_UNKNOWN
fi
 
 
REMOTE_TIME=$(date -d "$($NET_BIN time -I $IPADDR)" +%s)
LOCAL_TIME=$(date +%s)
DIFF_SEC=$(($LOCAL_TIME-$REMOTE_TIME))
DIFF=$(echo $DIFF_SEC|tr -d "-")
 
 
if (( $DIFF >= $CRITICAL ))
  then
    OUT=CRITICAL
    RETVAL=$STATE_CRITICAL
 
elif (( $DIFF >= $WARNING ))
  then
    OUT=WARNING
    RETVAL=$STATE_CRITICAL
 
else
  OUT=OK
  RETVAL=$STATE_OK
fi
 
 
RETSTR="TIMESYNC $OUT: Offset $DIFF_SEC secs - Timesync checked via Host $HOSTNAME|offset=${DIFF_SEC}s;$WARNING;$CRITICAL;"
 
 
echo $RETSTR
exit $RETVAL
